using System;
using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using SimpleJSON;

namespace ScaleReceiver {
	public class ScaleReceiver : MVRScript {

#region Vars
//////////////////////////////////////////////////////////
//////////////////// CONSTANTS ///////////////////////////
//////////////////////////////////////////////////////////
	    protected const float initFramesPerUpdate = 4.0f;
	    protected const float framesPerUpdateMin = 1.0f;
	    protected const float framesPerUpdateMax = 10.0f;

	    protected const float personScaleMin = 0.06f;  
		protected const float personScaleMax = 20f;
	    
	    protected const float worldScaleMin = 0.06f;  
		protected const float worldScaleMax = 20f;
//////////////////////////////////////////////////////////

		private Atom person;
		private JSONStorableFloat personScale;
		private JSONStorableFloat pScale;
		private JSONStorableFloat framesPerUpdate;

		private JSONStorableBool syncWorldScale;
		protected UIDynamicSlider personScaleSlider;
		protected UIDynamicSlider framesPerUpdateSlider;

		private int fpu;
	    private int frameCount;

		private JSONStorableFloat wScale;
		protected UIDynamicSlider worldScaleSlider;

		protected float initScale;
#endregion

		public override void Init() {
			try {				
				// Initialize script mode
				if (containingAtom.type == "Person") {
					pluginLabelJSON.val = "ScaleReceiver (person mode)";
					person = containingAtom;
					JSONStorable scaleReceiver = person.GetStorableByID("rescaleObject");
					if (scaleReceiver) {
						personScale = scaleReceiver.GetFloatJSONParam("scale");
					}

					initScale = personScale.val;
	                pScale = new JSONStorableFloat("Person Scale", initScale, SetPersonScale, personScaleMin, personScaleMax, true);
	                RegisterFloat(pScale);
	                CreateSlider(pScale, true);					

					framesPerUpdate = new JSONStorableFloat("Frames Per Update", initFramesPerUpdate, SetFramesPerUpdate, framesPerUpdateMin, framesPerUpdateMax, true);
					RegisterFloat(framesPerUpdate);
					framesPerUpdateSlider = CreateSlider(framesPerUpdate, true);

					syncWorldScale = new JSONStorableBool("Sync World Scale", false);
					CreateToggle(syncWorldScale, true);
					RegisterBool(syncWorldScale);

                } else {
					pluginLabelJSON.val = "ScaleReceiver (world mode)";
					initScale = SuperController.singleton.worldScale;
	                wScale = new JSONStorableFloat("World Scale", initScale, SetWorldScale, worldScaleMin, worldScaleMax, true);
	                RegisterFloat(wScale);
	                CreateSlider(wScale, true);
				}
				
				fpu = (int) Mathf.Floor(initFramesPerUpdate);
				frameCount = 0;
			}
			catch (Exception e) {
				SuperController.LogError("Exception caught: " + e);
			}
		}

		// Start is called once before Update or FixedUpdate is called and after Init()
		void Start() {
			try {
				// put code in here
			}
			catch (Exception e) {
				SuperController.LogError("Exception caught: " + e);
			}
		}

		// Update is called with each rendered frame by Unity
		void Update() {
			try {
				// put code in here
			}
			catch (Exception e) {
				SuperController.LogError("Exception caught: " + e);
			}
		}

// FixedUpdate is called with each physics simulation frame by Unity
		void FixedUpdate() {
			try {
				if (person) {
					if (frameCount % fpu == 0) {
						SetPersonScale(pScale);
						if (syncWorldScale.val) {
							SetWorldScale(pScale);
						}	
					}
					frameCount++;
				} else {
					SetWorldScale(wScale);
				}	
			}

			catch (Exception e) {
				SuperController.LogError("Exception caught: " + e);
			}
		}

		// OnDestroy is where you should put any cleanup
		// if you registered objects to supercontroller or atom, you should unregister them here
		void OnDestroy() {
		}

#region Functions
        protected void SetWorldScale(JSONStorableFloat jf) {
        	if (jf.val < worldScaleMin) {
        		SuperController.singleton.worldScale = worldScaleMin;
        	} else if (jf.val > worldScaleMax) {
            	SuperController.singleton.worldScale = worldScaleMax;
        	} else {
        		SuperController.singleton.worldScale = jf.val;
        	}
        }

        protected void SetPersonScale(JSONStorableFloat jf) {
        	if (jf.val < personScaleMin) {
        		personScale.val = personScaleMin;
        	} else if (jf.val > personScaleMax) {
        		personScale.val = personScaleMax;
        	} else {
        		personScale.val = jf.val;
        	}
        }

		protected void SetFramesPerUpdate(JSONStorableFloat jf) {
			fpu = (int) Mathf.Floor(Mathf.Max(1.0f, jf.val));
		}
#endregion

    }
}